#include "string_view1.h"
#include <catch.h>
#include <cstring>
#include <sstream>

static const char* const hello0world = "hello\0world";

TEST_CASE("range constructor")
{
    string_view sv1 {hello0world, hello0world + std::strlen(hello0world)};
    string_view sv2 {hello0world, hello0world + 11};
    CHECK(sv1.size() == 5);
    CHECK(sv2.size() == 11);
}

TEST_CASE("crossed range becomes empty")
{
    string_view sv {hello0world + 3, hello0world};
    CHECK( sv.size() == 0 );
}

TEST_CASE("start and size constructor")
{
    string_view sv1 {hello0world, std::strlen(hello0world)};
    string_view sv2 {hello0world, 11};
    CHECK(sv1.size() == 5);
    CHECK(sv2.size() == 11);
}

TEST_CASE("C style")
{
    string_view sv {hello0world};
    CHECK(sv.size() == 5);
}

TEST_CASE("stream insertion (printing)")
{
    string_view sv1("hello");
    string_view sv2(hello0world, 11);

    std::ostringstream oss;
    oss << sv1;
    CHECK( oss.str() == "hello" );

    oss.str("");
    oss << sv2;
    CHECK( oss.str() == std::string(hello0world, 11) );
}
